/*
 *  Base64Decoder.java
 *  Copyright (C) 2000  P. Taylor Goetz <tgoetz@fast.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package catscratch;

/** Utility class for decoding Base64-encoded strings.
 */
public class Base64Decoder {

    public static byte[] decode(String src) {
        int prePad = (src.length() % 4);
        int pad = 0;
        int srcLen = src.length();
        // pad w/'=' to make divisible by 4
        if(prePad != 0) {
            prePad = 4 - prePad;
            //System.out.println("padding input: " + prePad);
            for(int i=0;i<prePad;i++){
                src += "=";
            }
        }
        for (int i = srcLen - 1; src.charAt(i) == '='; i--){
            pad++;
        }
        int length = srcLen * 6 / 8 - pad;
        int index = 0;
        byte[] retVal = new byte[length];
        for (int i = 0; i < src.length(); i += 4) {
            int block = (getValue(src.charAt(i)) << 18)
            + (getValue(src.charAt(i + 1)) << 12)
            + (getValue(src.charAt(i + 2)) << 6)
            + (getValue(src.charAt(i + 3)));
            for (int j = 0; j < 3 && index + j < retVal.length; j++)
            retVal[index + j] = (byte)((block >> (8 * (2 - j))) & 0xff);
            index += 3;
        }
        return retVal;
    }

    protected static int getValue(char c) {
        if (c >= 'A' && c <= 'Z') return c - 'A';
        if (c >= 'a' && c <= 'z') return c - 'a' + 26;
        if (c >= '0' && c <= '9') return c - '0' + 52;
        if (c == '+') return 62;
        if (c == '/') return 63;
        if (c == '=') return 0;
        return -1;
    }
}